/*
 * ALSA Patch Bay
 *
 * Copyright (C) 2002 Robert Ham (node@users.sourceforge.net)
 *
 * You have permission to use this file under the GNU General
 * Public License, version 2 or later.  See the file COPYING
 * for the full text.
 *
 */

#ifndef __APB_APB_H__
#define __APB_APB_H__

#include <list>
#include <string>
#include <vector>

#define DEBUG_STRING string(__FILE__) + ":" + Misc::int2string(__LINE__) + ":" + \
      __FUNCTION__ + ": "
      
      
namespace APB {

typedef enum { ERROR = 0,
               WARNING,
               NOTICE,
               DEBUG } DebugLevel;

struct Exception {
  string desc;
  int err;
  Exception (string d) : desc (d) {}
  Exception (string d, int e) : desc (d), err (e) {}
};

class Misc {
  public:
    static string int2string (int);
    static int randomNumber (double, double);
};


/***********************************************/
/************** Driver interface ***************/
/***********************************************/

typedef enum { ALSA, JACK } DriverType;

class Driver;

class Addr {
  public:
    virtual bool equals (const Addr *) const = 0;
    virtual Driver * driver () const = 0;
    virtual string getName () const = 0;
};

class Subscription {
  private:
    const Addr * _from;
    const Addr * _to;
  public:
    Subscription (const Addr * from, const Addr * to)
      : _from (from), _to (to) {}
    Subscription (const Subscription& subscription)
      : _from (subscription._from),
        _to (subscription._to) {}
    const Addr * from () const { return _from; }
    const Addr * to () const { return _to; }
    string getName () {
      return string("'") + _from->getName() + "' -> '" + _to->getName() + "'";
    }
    bool operator == (const Subscription& sub) {
      return sub.to()->equals (_to) &&
             sub.from()->equals (_from);
    }
};

class UI;

class Driver {
  public:
    virtual string findClientName (const Addr *) const = 0;
    virtual string findPortName (const Addr *) const = 0;
    virtual const list<Addr *>& getReadPorts (UI&) = 0;
    virtual const list<Addr *>& getWritePorts (UI&) = 0;
    virtual const list<Subscription>& getSubscriptions (UI&) = 0;
    virtual void refreshPorts (UI&) = 0;
    virtual void refreshSubscriptions (UI&) = 0;
    virtual void subscribePorts (UI&, const Addr *, const Addr *) = 0;
    virtual void unsubscribePorts (UI&, const Addr *, const Addr *) = 0;
    virtual void setJFD (bool) = 0;
};


/*******************************************/
/************** UI interface ***************/
/*******************************************/

typedef enum { GTKMM } UIType;

typedef enum { ONLYONE, ANY } ChoiceType;

struct Choice {
//  int _id;
  string _choice;
  bool _takesInt;
  int _value;
  Choice (//int id,
          const string& choice,
          bool takesInt = false,
          int defaultValue = 0)
    : //_id (id),
      _choice (choice),
      _takesInt (takesInt),
      _value (defaultValue) {}
  Choice (const Choice& choice)
    : //_id (choice._id),
      _choice (choice._choice),
      _takesInt (choice._takesInt),
      _value (choice._value) {}
};

struct ChoiceSpec {
  string _message;
  ChoiceType _type;
  list<Choice> _choices;
  ChoiceSpec (string message, ChoiceType type)
    : _message (message), _type (type) {}
  void addChoice (const Choice& choice) {
    _choices.push_back (choice);
  }
};

typedef pair< bool, int > ChoiceResult; // (selected, value)
                                              	    	  	

class UI {
  public:
    virtual void log (DebugLevel, const string&) = 0;
    virtual void run () = 0;
    virtual vector<ChoiceResult> getChoice (string, const ChoiceSpec&) = 0;
};

/*******************************************/
/************** AlsaPatchBay ***************/
/*******************************************/

class AlsaPatchBay {
  private:
    UI * _ui;
    Driver * _driver;
    static Driver * newDriver (string, int, char **, DriverType, void *);
    static UI * newUI (string, int, char **, UIType, void *, Driver *);
  public:
    AlsaPatchBay (string title, int argc, char ** argv,
                  DriverType driverType = ALSA, void * driverData = 0,
                  UIType uiType = GTKMM, void * uiData = 0) {
      _driver = newDriver (title, argc, argv, driverType, driverData);
      _ui = newUI (title, argc, argv, uiType, uiData, _driver);
    }
    void run () {
      _ui->run ();
    }
};

} /* namespace APB */

extern APB::DebugLevel debugLevel;

#endif /* __APB_APB_H__ */
